package com.gyee.common.util;

import org.springframework.http.HttpEntity;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpMethod;
import org.springframework.http.ResponseEntity;
import org.springframework.web.client.RestTemplate;

/**
 * @ClassName : RestTemplateUtil
 * @Author : xieshengjie
 * @Date: 2022/2/10 14:15
 * @Description :
 */
public class RestTemplateUtil {

    private static class SingletonRestTemplate {
        static final RestTemplate INSTANCE = new RestTemplate();
    }

    private RestTemplateUtil() {

    }

    public static RestTemplate getInstance() {
        return SingletonRestTemplate.INSTANCE;
    }

    /**
     * * GET请求
     *
     * @param url   请求路径
     * @param token jwt所需的Token不需要可传null
     * @return
     */
    public static String get(String url, String token) {
        HttpHeaders headers = new HttpHeaders();
        headers.add("Accept", "application/json");
        headers.add("Content-Encoding", "UTF-8");
        headers.add("Content-Type", "application/json; charset=UTF-8");
        if (token != null) {
            headers.add("token", token);
        }
        HttpEntity<String> requestEntity = new HttpEntity<>(null, headers);
        ResponseEntity<String> response = RestTemplateUtil.getInstance().exchange(url, HttpMethod.GET, requestEntity, String.class);
        String responseInfo = response.getBody();
        return responseInfo;
    }

    /**
     * * POST请求
     *
     * @param url   请求路径
     * @param data  body数据
     * @param token jwt所需的Token不需要可传null
     * @return
     */
    public static String post(String url, String data, String token) {
        HttpHeaders headers = new HttpHeaders();
        headers.add("Accept", "application/json");
        headers.add("Content-Encoding", "UTF-8");
        headers.add("Content-Type", "application/json; charset=UTF-8");
        if (token != null) {
            headers.add("Authorization", token);
        }
        HttpEntity<String> requestEntity = new HttpEntity<>(data, headers);
        return RestTemplateUtil.getInstance().postForObject(url, requestEntity, String.class);
    }


    public static void main(String[] args) {
        //        //Get接口调用
        //        String url="http://www.kuaidi100.com/query?type=yuantong&postid=11111111111";
        //        String responseInfo = RestTemplateUtil.get(url, null);
        //        System.out.println(responseInfo);
        //
        //        //Post接口调用
        //        String Url="http://localhost:8080/login/doLogin";
        //        ObjectMapper objectMapper = new ObjectMapper();
        //        String data = null;
        //        UserLogin userLogin=new UserLogin("clover","12345","weq");
        //        try {
        //            data = objectMapper.writeValueAsString(userLogin);
        //        } catch (JsonProcessingException e) {
        //            e.printStackTrace();
        //        }
        //        String responseInfo2 = RestTemplateUtil.post(Url, data, null);
        //        System.out.println(responseInfo2);
    }
}
